#!/usr/bin/env python

"""Tests for `ghoclient` package."""

import unittest
from click.testing import CliRunner

import ghoclient
from ghoclient import cli
from ghoclient import Index
import pandas as pd
from whoosh.searching import Hit


class TestGhoclient(unittest.TestCase):
    """Tests for `ghoclient` package."""

    def setUp(self):
        """Set up test fixtures, if any."""

    def tearDown(self):
        """Tear down test fixtures, if any."""

    def test_000_something(self):
        """Test something."""

    def test_command_line_interface(self):
        """Test the CLI."""
        runner = CliRunner()
        result = runner.invoke(cli.main)
        assert result.exit_code == 0
        # assert 'ghoclient.cli.main' in result.output
        help_result = runner.invoke(cli.main, ['--help'])
        assert help_result.exit_code == 0
        assert '--help  Show this message and exit.' in help_result.output


class TestGHO(unittest.TestCase):
    def test_get_countries_as_df(self):
        GC = ghoclient.ghoclient.GHOSession()
        df = GC.get_countries()
        self.assertIsInstance(df, pd.DataFrame)

    def test_get_dimensions_as_df(self):
        GC = ghoclient.ghoclient.GHOSession()
        df = GC.get_dimensions()
        self.assertIsInstance(df, pd.DataFrame)
        self.assertEquals(len(df.columns), 3)


class Test_Index(unittest.TestCase):
    def test_build_index(self):
        ghoclient.index.build_index(None)
        assert ghoclient.index.ix is not None 
        
    def test_search(self):
        res = ghoclient.index.search('tuberculosis')
        self.assertGreaterEqual(len(res), 0)
        self.assertIsInstance(res[0], dict)
        self.assertIn('code', res[0])


"""Console script for ghoclient."""
import sys
import click
from ghoclient import GHOSession


@click.command()
def main(args=None):
    """Console script for ghoclient."""

    return 0


if __name__ == "__main__":
    sys.exit(main())  # pragma: no cover



"""
Main module.
Provides a class with utility methods for fetching data from the Global Health Observatory.
"""

import xmltodict
import pandas as pd
import requests
from pprint import pprint

BASE_URL = 'http://apps.who.int/gho/athena/api/'


class GHOSession:
    def __init__(self):
        resp = requests.get(BASE_URL)
        self.data = xmltodict.parse(resp.text)

    def get_available_datasets(self):
        '''
        Check for available datasets

        Returns 
        -------
        datasets : list
            list of datasets.

        '''
        datasets = []
        for k, v in self.data['GHO']['Metadata'].items():
            if k == 'Dataset':
                datasets.extend([d for d in v])
        return datasets

    def get_attributes(self):
        '''
        Lists attributes used on datasets

        Returns
        -------
        attributes : list
            list of attributes represented as dictionaries.

        '''
        attributes = []
        for k, v in self.data['GHO']['Metadata'].items():
            if k == 'Attribute':
                attributes.extend([d for d in v])
        return attributes

    def get_dimensions(self, format='dataframe'):
        '''
        List dimensions of data

        Parameters
        ----------
        format : str, output format, of `dataframe` (default) or `list`
            DESCRIPTION. The default is 'dataframe'.

        Returns
        -------
        dimensions: dataframe or list
            description of every variable in the datasets and their dimension.

        '''
        dimensions = []
        for k, v in self.data['GHO']['Metadata'].items():
            if k == 'Dimension':
                dimensions.extend([d for d in v])
        if format == 'dataframe':
            return pd.DataFrame(dimensions)
        return dimensions

    def get_region_codes(self):
        '''
        Returns region codes

        Returns
        -------
        regions : dictionary
            Dictionary with code, description pairs.

        '''
        url = BASE_URL + 'REGION'
        data = self._fetch_data_as_dict(url)
        regions = {c['@Label']: c['Display'] for c in data['GHO']['Metadata']['Dimension']['Code']}
        return regions

    def get_countries(self, format='dataframe'):
        '''
        Returns a dataframe with country codes and metadata

        Parameters
        ----------
        format : str, optional
            The default is 'dataframe'.

        Returns list if `format` is full
        -------
        data: dataframe by default
            country info.

        '''
        url = BASE_URL + 'COUNTRY'
        data = self._fetch_data_as_dict(url)
        if format == 'dataframe':
            lines = []
            for d in data['GHO']['Metadata']['Dimension']['Code']:
                rec = {k: v for k, v in d.items() if k != 'Attr'}
                if 'Attr' in d:
                    for attr in d['Attr']:
                        rec[attr['@Category']] = attr['Value']['Display']
                lines.append(rec)
            return pd.DataFrame(lines)
        elif format == 'full':
            return data['GHO']['Metadata']['Dimension']['Code']
        return data

    def fetch_data_from_codes(self, code=None, like='MALARIA'):
        """
        Fetches data for a specific indicator code or a list of indicators matching the substring in `like`
        :param code: Indicator code to fetch (for a full list of available codes use `get_data_codes` method.)
        :param like: substring of the codes desired
        :return: Dataframe with table.
        """
        url = BASE_URL + 'GHO/'
        if code is None:
            codes = [c for c in self.get_data_codes(format='label') if like.lower() in c.lower()]
        
        elif isinstance(code, (list, tuple)):
            codes = list(code)
        elif isinstance(code, str):
            codes=[code]
                
        url += ','.join(codes)
        url += '&format=csv' if '?' in url else '?format=csv'
        data = pd.read_csv(url)
        # data = requests.get(url)
        return data



    def _fetch_data_as_dict(self, url):
        """
        Downloads XML data, parses it and return as dict.
        :param url:
        """
        resp = requests.get(url)
        data = xmltodict.parse(resp.text)
        return data

    def get_data_codes(self, format='dataframe'):
        """
        Get Codes that can be fetched as indicators.
        :param format: either 'full', 'label' or 'url'
        :return: list of dicts when `format` is 'full', a Dataframe when it is 'dataframe' or a list of strings otherwise.
        """
        url = BASE_URL + 'GHO'
        data = self._fetch_data_as_dict(url)
        # codes = [d for d in data['GHO']['Metadata']['Dimension']['Code']]
        if format == 'full':
            return [d for d in data['GHO']['Metadata']['Dimension']['Code']]
        if format == 'dataframe':
            return pd.DataFrame([d for d in data['GHO']['Metadata']['Dimension']['Code']])
        elif format == 'label':
            return [d['@Label'] for d in data['GHO']['Metadata']['Dimension']['Code']]
        elif format == 'url':
            return [d['@URL'] for d in data['GHO']['Metadata']['Dimension']['Code']]

if __name__ == "__main__":
    GC = GHOSession()

    # pprint(GC.get_available_datasets())
    # print(len(GC.get_available_datasets()))
    # pprint(GC.get_attributes())
    # pprint(GC.get_dimensions())
    pprint(GC.get_data_codes(format='dataframe'))

    # pprint(GC.get_region_codes())
    # print(GC.get_countries())
    # pprint(GC.fetch_data_from_codes())



from whoosh.index import create_in, open_dir
from whoosh.qparser import QueryParser
from whoosh.fields import * 
import os
import copy

schema = Schema(code=ID(stored=True), description=TEXT(stored=True))


class Index:
    def __init__(self, index_path='indexdir'):
        if not os.path.exists(index_path):
            os.mkdir(index_path)
            self.ix = None
        else:
            self.ix = open_dir(index_path)
        
    def build_index(self, codes):
        if self.ix is None:
            self.ix = create_in("indexdir", schema)
            writer = self.ix.writer()
            for row in codes.itertuples():
                writer.add_document(code=row.Label, description=row.Display)
            writer.commit()
            
        
    def search(self, query):
        with self.ix.searcher() as searcher:
            qp = QueryParser("description", schema=self.ix.schema)
            q = qp.parse(query)
            results = searcher.search(q)
            out  = [dict(h) for h in results]
        return out
        
        



"""Top-level package for GHOclient."""

__author__ = """Flávio Codeço Coelho"""
__email__ = 'fccoelho@gmail.com'
__version__ = '0.1.0'

from .ghoclient import GHOSession
from ghoclient.index import Index

index = Index()
GC = GHOSession()
datacodes =  GC.get_data_codes(format='dataframe')
datacodes.columns = [c.strip('@') for c in datacodes.columns]
index.build_index(datacodes)



